@@ -984,8 +984,9 @@ necessary, since type inference may take arbitrarily long to converge.")
                        'input-error-in-compile-file)
     (with-source-paths
       (find-source-paths form current-index)
-      (process-toplevel-form
-       form `(original-source-start 0 ,current-index) nil)))
+      (let ((sb!xc:*gensym-counter* 0))
+        (process-toplevel-form
+         form `(original-source-start 0 ,current-index) nil))))
   ;; It's easy to get into a situation where cold-init crashes and the only
   ;; backtrace you get from ldb is TOP-LEVEL-FORM, which means you're anywhere
   ;; within the 23000 or so blobs of code deferred until cold-init.
@@ -1507,13 +1508,23 @@ necessary, since type inference may take arbitrarily long to converge.")
               (value-forms call)
               (values))
           (when (and (every #'symbolp slot-names)
-                     (every #'sb!xc:constantp value-forms))
-            (dolist (x value-forms)
-              (let ((val (constant-form-value x)))
-                ;; invoke recursive MAKE-LOAD-FORM stuff as necessary
-                (find-constant val)
-                (push val values)))
-            (mapc (lambda (x) (dump-object x fasl)) (nreverse values))
+                     (every (lambda (x)
+                              ;; +SLOT-UNBOUND+ is not a constant,
+                              ;; but is trivially dumpable.
+                              (or (eql x 'sb!pcl:+slot-unbound+)
+                                  (sb!xc:constantp x)))
+                            value-forms))
+            (dolist (form value-forms)
+              (unless (eq form 'sb!pcl:+slot-unbound+)
+                (let ((val (constant-form-value form)))
+                  ;; invoke recursive MAKE-LOAD-FORM stuff as necessary
+                  (find-constant val)
+                  (push val values))))
+            (setq values (nreverse values))
+            (dolist (form value-forms)
+              (if (eq form 'sb!pcl:+slot-unbound+)
+                  (dump-fop 'sb!fasl::fop-misc-trap fasl)
+                  (dump-object (pop values) fasl)))
             (dump-object (cons (length slot-names) slot-names) fasl)
             (dump-object instance fasl)
             (dump-fop 'sb!fasl::fop-set-slot-values fasl)
