@@ -97,7 +97,8 @@
     (when old-method
       (remove-method #'find-method-combination old-method))
     (add-method #'find-method-combination new-method)
-    (setf (random-documentation type-name 'method-combination) doc)
+    (when doc
+      (setf (random-documentation type-name 'method-combination) doc))
     type-name))
 
 (defun short-combine-methods (type-name options operator ioa method doc)
@@ -118,90 +119,29 @@
                  :definition-source method
                  :documentation doc))
 
-(defmethod compute-effective-method ((generic-function generic-function)
-                                     (combin short-method-combination)
-                                     applicable-methods)
-  (let ((type-name (method-combination-type-name combin))
-        (operator (short-combination-operator combin))
-        (ioa (short-combination-identity-with-one-argument combin))
-        (order (car (method-combination-options combin)))
-        (around ())
-        (primary ()))
-    (flet ((invalid (gf combin m)
-             (return-from compute-effective-method
-               `(%invalid-qualifiers ',gf ',combin ',m))))
-      (dolist (m applicable-methods)
-        (let ((qualifiers (method-qualifiers m)))
-          (cond ((null qualifiers) (invalid generic-function combin m))
-                ((cdr qualifiers) (invalid generic-function combin m))
-                ((eq (car qualifiers) :around)
-                 (push m around))
-                ((eq (car qualifiers) type-name)
-                 (push m primary))
-                (t (invalid generic-function combin m))))))
-    (setq around (nreverse around))
-    (ecase order
-      (:most-specific-last) ; nothing to be done, already in correct order
-      (:most-specific-first
-       (setq primary (nreverse primary))))
-    (let ((main-method
-            (if (and (null (cdr primary))
-                     (not (null ioa)))
-                `(call-method ,(car primary) ())
-                `(,operator ,@(mapcar (lambda (m) `(call-method ,m ()))
-                                      primary)))))
-      (cond ((null primary)
-             ;; As of sbcl-0.8.0.80 we don't seem to need to need
-             ;; to do anything messy like
-             ;;        `(APPLY (FUNCTION (IF AROUND
-             ;;                              'NO-PRIMARY-METHOD
-             ;;                              'NO-APPLICABLE-METHOD)
-             ;;                           ',GENERIC-FUNCTION
-             ;;                           .ARGS.)
-             ;; here because (for reasons I don't understand at the
-             ;; moment -- WHN) control will never reach here if there
-             ;; are no applicable methods, but instead end up
-             ;; in NO-APPLICABLE-METHODS first.
-             ;;
-             ;; FIXME: The way that we arrange for .ARGS. to be bound
-             ;; here seems weird. We rely on EXPAND-EFFECTIVE-METHOD-FUNCTION
-             ;; recognizing any form whose operator is %NO-PRIMARY-METHOD
-             ;; as magical, and carefully surrounding it with a
-             ;; LAMBDA form which binds .ARGS. But...
-             ;;   1. That seems fragile, because the magicalness of
-             ;;      %NO-PRIMARY-METHOD forms is scattered around
-             ;;      the system. So it could easily be broken by
-             ;;      locally-plausible maintenance changes like,
-             ;;      e.g., using the APPLY expression above.
-             ;;   2. That seems buggy w.r.t. to MOPpish tricks in
-             ;;      user code, e.g.
-             ;;         (DEFMETHOD COMPUTE-EFFECTIVE-METHOD :AROUND (...)
-             ;;           `(PROGN ,(CALL-NEXT-METHOD) (INCF *MY-CTR*)))
-             `(%no-primary-method ',generic-function .args.))
-            ((null around) main-method)
-            (t
-             `(call-method ,(car around)
-                           (,@(cdr around) (make-method ,main-method))))))))
-
 (defmethod invalid-qualifiers ((gf generic-function)
                                (combin short-method-combination)
                                method)
-  (let ((qualifiers (method-qualifiers method))
-        (type-name (method-combination-type-name combin)))
-    (let ((why (cond
-                 ((null qualifiers) "has no qualifiers")
-                 ((cdr qualifiers) "has too many qualifiers")
-                 (t (aver (and (neq (car qualifiers) type-name)
-                               (neq (car qualifiers) :around)))
-                    "has an invalid qualifier"))))
-      (invalid-method-error
-       method
-       "The method ~S on ~S ~A.~%~
-        The method combination type ~S was defined with the~%~
-        short form of DEFINE-METHOD-COMBINATION and so requires~%~
-        all methods have either the single qualifier ~S or the~%~
-        single qualifier :AROUND."
-       method gf why type-name type-name))))
+  (let* ((qualifiers (method-qualifiers method))
+         (qualifier (first qualifiers))
+         (type-name (method-combination-type-name combin))
+         (why (cond
+                ((null qualifiers)
+                 "has no qualifiers")
+                ((cdr qualifiers)
+                 "has too many qualifiers")
+                (t
+                 (aver (not (short-method-combination-qualifier-p
+                             type-name qualifier)))
+                 "has an invalid qualifier"))))
+    (invalid-method-error
+     method
+     "~@<The method ~S on ~S ~A.~
+      ~@:_~@:_~
+      The method combination type ~S was defined with the short form ~
+      of DEFINE-METHOD-COMBINATION and so requires all methods have ~
+      either ~{the single qualifier ~S~^ or ~}.~@:>"
+     method gf why type-name (short-method-combination-qualifiers type-name))))
 
 ;;;; long method combinations
 
@@ -269,7 +209,7 @@
        (type-name ll method-group-specifiers args-option gf-var body)
   (declare (ignore type-name))
   (multiple-value-bind (real-body declarations documentation)
-      (parse-body body)
+      (parse-body body t)
     (let ((wrapped-body
             (wrap-method-group-specifier-bindings method-group-specifiers
                                                   declarations
@@ -296,7 +236,7 @@
     (reference-condition simple-error)
   ()
   (:default-initargs
-      :references (list '(:ansi-cl :macro define-method-combination))))
+   :references (list '(:ansi-cl :macro define-method-combination))))
 
 ;;; NOTE:
 ;;;
@@ -545,4 +485,3 @@
             (return (nconc (frob required nr nreq)
                            (frob optional no nopt)
                            values)))))
-
