@@ -385,7 +385,8 @@ static lispobj
 trans_fun_header(lispobj object)
 {
     struct simple_fun *fheader = (struct simple_fun *) native_pointer(object);
-    uword_t offset = HeaderValue(fheader->header) * N_WORD_BYTES;
+    uword_t offset =
+        (HeaderValue(fheader->header) & FUN_HEADER_NWORDS_MASK) * N_WORD_BYTES;
 
     /* Transport the whole code object */
     struct code *code = (struct code *) ((uword_t) fheader - offset);
@@ -754,8 +755,7 @@ trans_vector(lispobj object)
 {
     gc_dcheck(lowtag_of(object) == OTHER_POINTER_LOWTAG);
 
-    sword_t length =
-        fixnum_value(((struct vector*)native_pointer(object))->length);
+    sword_t length = fixnum_value(VECTOR(object)->length);
     return copy_large_object(object, CEILING(length + 2, 2));
 }
 
@@ -792,7 +792,7 @@ NWORDS(uword_t x, uword_t n_bits)
   } \
   static lispobj __attribute__((unused)) trans_##name(lispobj object) { \
     gc_dcheck(lowtag_of(object) == OTHER_POINTER_LOWTAG); \
-    sword_t length = fixnum_value(((struct vector*)(object-OTHER_POINTER_LOWTAG))->length); \
+    sword_t length = fixnum_value(VECTOR(object)->length); \
     return copy_large_unboxed_object(object, CEILING(nwords + 2, 2)); \
   } \
   static sword_t __attribute__((unused)) size_##name(lispobj *where) { \
@@ -930,45 +930,34 @@ static inline int pointer_survived_gc_yet(lispobj obj)
 #endif
 }
 
-#ifdef EMPTY_HT_SLOT /* only if it's a static symbol */
-// "ish" because EMPTY_HT_SLOT is of course a pointer.
-#  define ht_cell_nonpointerish(x) (!is_lisp_pointer(x) || x==EMPTY_HT_SLOT)
-#else
-#  define ht_cell_nonpointerish(x) !is_lisp_pointer(x)
-#endif
-
 static int survived_gc_yet_KEY(lispobj key, lispobj value) {
-    return ht_cell_nonpointerish(key) || pointer_survived_gc_yet(key);
+    return !is_lisp_pointer(key) || pointer_survived_gc_yet(key);
 }
 static int survived_gc_yet_VALUE(lispobj key, lispobj value) {
-    return ht_cell_nonpointerish(value) || pointer_survived_gc_yet(value);
+    return !is_lisp_pointer(value) || pointer_survived_gc_yet(value);
 }
 static int survived_gc_yet_AND(lispobj key, lispobj value) {
-    int key_nonpointer = ht_cell_nonpointerish(key);
-    int val_nonpointer = ht_cell_nonpointerish(value);
+    int key_nonpointer = !is_lisp_pointer(key);
+    int val_nonpointer = !is_lisp_pointer(value);
     if (key_nonpointer && val_nonpointer) return 1;
     return (key_nonpointer || pointer_survived_gc_yet(key))
         && (val_nonpointer || pointer_survived_gc_yet(value));
 }
 static int survived_gc_yet_OR(lispobj key, lispobj value) {
-    int key_nonpointer = ht_cell_nonpointerish(key);
-    int val_nonpointer = ht_cell_nonpointerish(value);
+    int key_nonpointer = !is_lisp_pointer(key);
+    int val_nonpointer = !is_lisp_pointer(value);
     if (key_nonpointer || val_nonpointer) return 1;
     // Both MUST be pointers
     return pointer_survived_gc_yet(key) || pointer_survived_gc_yet(value);
 }
 
-static int (*weak_hash_entry_alivep_fun(lispobj weakness))(lispobj,lispobj)
-{
-    switch (weakness) {
-    case KEY: return survived_gc_yet_KEY;
-    case VALUE: return survived_gc_yet_VALUE;
-    case KEY_OR_VALUE: return survived_gc_yet_OR;
-    case KEY_AND_VALUE: return survived_gc_yet_AND;
-    case NIL: return NULL;
-    default: lose("Bad hash table weakness");
-    }
-}
+static int (*weak_hash_entry_alivep_fun[5])(lispobj,lispobj) = {
+    NULL,
+    survived_gc_yet_KEY,
+    survived_gc_yet_VALUE,
+    survived_gc_yet_AND,
+    survived_gc_yet_OR
+};
 
 /* Return the beginning of data in ARRAY (skipping the header and the
  * length) or NULL if it isn't an array of the specified widetag after
@@ -1002,7 +991,6 @@ scav_hash_table_entries (struct hash_table *hash_table)
     lispobj *hash_vector;
     uword_t hash_vector_length;
     lispobj empty_symbol;
-    lispobj weakness = hash_table->weakness;
     uword_t i;
 
     kv_vector = get_array_data(hash_table->table,
@@ -1032,15 +1020,12 @@ scav_hash_table_entries (struct hash_table *hash_table)
       * help reduce collisions. */
      gc_assert(next_vector_length*2 == kv_length);
 
-    empty_symbol = kv_vector[1];
-    /* fprintf(stderr,"* empty_symbol = %x\n", empty_symbol);*/
-    if (widetag_of(*native_pointer(empty_symbol)) != SYMBOL_WIDETAG) {
-        lose("not a symbol where empty-hash-table-slot symbol expected: %x\n",
-             *native_pointer(empty_symbol));
-    }
+     if ((empty_symbol = kv_vector[1]) != UNBOUND_MARKER_WIDETAG)
+        lose("unexpected empty-hash-table-slot marker: %p\n", empty_symbol);
 
     /* Work through the KV vector. */
-    int (*alivep_test)(lispobj,lispobj) = weak_hash_entry_alivep_fun(weakness);
+    int (*alivep_test)(lispobj,lispobj)
+        = weak_hash_entry_alivep_fun[fixnum_value(hash_table->_weakness)];
 #define SCAV_ENTRIES(aliveness_predicate) \
     for (i = 1; i < next_vector_length; i++) {                                 \
         lispobj old_key = kv_vector[2*i];                                      \
@@ -1098,12 +1083,10 @@ scav_vector (lispobj *where, lispobj object)
              hash_table);
     }
 
-    /* Scavenge element 1, which should be some internal symbol that
-     * the hash table code reserves for marking empty slots. */
-    scavenge(where+3, 1);
-    if (!is_lisp_pointer(where[3])) {
-        lose("not empty-hash-table-slot symbol pointer: %x\n", where[3]);
-    }
+    /* Verify that vector element 1 is as expected.
+       Don't bother scavenging it, since we lose() if it's not an immediate. */
+    if (where[3] != UNBOUND_MARKER_WIDETAG)
+        lose("unexpected empty-hash-table-slot marker: %p\n", where[3]);
 
     /* Scavenge hash table, which will fix the positions of the other
      * needed objects. */
@@ -1114,7 +1097,7 @@ scav_vector (lispobj *where, lispobj object)
         lose("hash_table table!=this table %x\n", hash_table->table);
     }
 
-    if (hash_table->weakness == NIL) {
+    if (!hash_table->_weakness) {
         scav_hash_table_entries(hash_table);
     } else {
         /* Delay scavenging of this table by pushing it onto
@@ -1184,7 +1167,8 @@ scan_weak_hash_table (struct hash_table *hash_table)
     uword_t next_vector_length = 0; /* prevent warning */
     lispobj *hash_vector;
     lispobj empty_symbol;
-    lispobj weakness = hash_table->weakness;
+    int (*alivep_test)(lispobj,lispobj) =
+        weak_hash_entry_alivep_fun[fixnum_value(hash_table->_weakness)];
     uword_t i;
 
     kv_vector = get_array_data(hash_table->table,
@@ -1201,8 +1185,7 @@ scan_weak_hash_table (struct hash_table *hash_table)
     for (i = 0; i < length; i++) {
         scan_weak_hash_table_chain(hash_table, &index_vector[i],
                                    kv_vector, index_vector, next_vector,
-                                   hash_vector, empty_symbol,
-                                   weak_hash_entry_alivep_fun(weakness));
+                                   hash_vector, empty_symbol, alivep_test);
     }
 }
 
@@ -1940,33 +1923,84 @@ void gc_heapsort_uwords(heap array, int length)
     }
 }
 
-//// Coalescing of constant strings for SAVE-LISP-AND-DIE
+//// Coalescing of constant vectors for SAVE-LISP-AND-DIE
 
-static void remap_string(lispobj* where, struct hopscotch_table* ht)
+static boolean coalescible_number_p(lispobj* where)
 {
-    lispobj obj = *where;
-    struct vector* s = (struct vector*)native_pointer(obj);
+    int widetag = widetag_of(*where);
+    return widetag == BIGNUM_WIDETAG
+        // Ratios and complex integers containing pointers to bignums don't work.
+        || ((widetag == RATIO_WIDETAG || widetag == COMPLEX_WIDETAG)
+            && fixnump(where[1]) && fixnump(where[2]))
+#ifndef LISP_FEATURE_64_BIT
+        || widetag == SINGLE_FLOAT_WIDETAG
+#endif
+        || widetag == DOUBLE_FLOAT_WIDETAG
+        || widetag == COMPLEX_SINGLE_FLOAT_WIDETAG
+        || widetag == COMPLEX_DOUBLE_FLOAT_WIDETAG;
+}
+
+/// Return true of fixnums, bignums, strings, symbols.
+/// Strings are considered eql-comparable,
+/// because they're coalesced before comparing.
+static boolean eql_comparable_p(lispobj obj)
+{
+    if (fixnump(obj) || obj == NIL) return 1;
+    if (lowtag_of(obj) != OTHER_POINTER_LOWTAG) return 0;
+    int widetag = widetag_of(*native_pointer(obj));
+    return widetag == BIGNUM_WIDETAG
+        || widetag == SYMBOL_WIDETAG
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
+        || widetag == SIMPLE_CHARACTER_STRING_WIDETAG
+#endif
+        || widetag == SIMPLE_BASE_STRING_WIDETAG;
+}
+
+static boolean vector_isevery(boolean (*pred)(lispobj), struct vector* v)
+{
+    int i;
+    for (i = fixnum_value(v->length)-1; i >= 0; --i)
+        if (!pred(v->data[i])) return 0;
+    return 1;
+}
+
+static void coalesce_obj(lispobj* where, struct hopscotch_table* ht)
+{
+    lispobj ptr = *where;
+    if (lowtag_of(ptr) != OTHER_POINTER_LOWTAG)
+        return;
+
     extern char gc_coalesce_string_literals;
     // gc_coalesce_string_literals represents the "aggressiveness" level.
-    // If 1, then we share strings tagged as +STRING-SHAREABLE+,
-    // but if >1, those and also +STRING-SHAREABLE-NONSTD+.
-    int mask = (gc_coalesce_string_literals>1 ? 6 : 2)<<N_WIDETAG_BITS;
+    // If 1, then we share vectors tagged as +VECTOR-SHAREABLE+,
+    // but if >1, those and also +VECTOR-SHAREABLE-NONSTD+.
+    int mask = gc_coalesce_string_literals > 1
+      ? (VECTOR_SHAREABLE|VECTOR_SHAREABLE_NONSTD)<<N_WIDETAG_BITS
+      : (VECTOR_SHAREABLE                        )<<N_WIDETAG_BITS;
 
-    if (lowtag_of(obj) == OTHER_POINTER_LOWTAG &&
-        (widetag_of(s->header) == SIMPLE_BASE_STRING_WIDETAG
-#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
-         || widetag_of(s->header) == SIMPLE_CHARACTER_STRING_WIDETAG
-#endif
-         ) && (s->header & mask) != 0) { /* shareable string */
-        int index = hopscotch_get(ht, (uword_t)s, 0);
+    lispobj* obj = native_pointer(ptr);
+    lispobj header = *obj;
+    int widetag = widetag_of(header);
+
+    if ((((header & mask) != 0) // optimistically assume it's a vector
+         && ((widetag == SIMPLE_VECTOR_WIDETAG
+              && vector_isevery(eql_comparable_p, (struct vector*)obj))
+             || specialized_vector_widetag_p(widetag)))
+        || coalescible_number_p(obj)) {
+        if (widetag == SIMPLE_VECTOR_WIDETAG) {
+            sword_t n_elts = fixnum_value(obj[1]), i;
+            for (i = 2 ; i < n_elts+2 ; ++i)
+                coalesce_obj(obj + i, ht);
+        }
+        int index = hopscotch_get(ht, (uword_t)obj, 0);
         if (!index) // Not found
-            hopscotch_insert(ht, (uword_t)s, 1);
+            hopscotch_insert(ht, (uword_t)obj, 1);
         else
             *where = make_lispobj((void*)ht->keys[index-1], OTHER_POINTER_LOWTAG);
     }
 }
 
-static uword_t remap_strings(lispobj* where, lispobj* limit, uword_t arg)
+static uword_t coalesce_range(lispobj* where, lispobj* limit, uword_t arg)
 {
     struct hopscotch_table* ht = (struct hopscotch_table*)arg;
     lispobj layout, bitmap, *next;
@@ -1975,8 +2009,8 @@ static uword_t remap_strings(lispobj* where, lispobj* limit, uword_t arg)
     for ( ; where < limit ; where = next ) {
         lispobj header = *where;
         if (is_cons_half(header)) {
-            remap_string(where+0, ht);
-            remap_string(where+1, ht);
+            coalesce_obj(where+0, ht);
+            coalesce_obj(where+1, ht);
             next = where + 2;
         } else {
             int widetag = widetag_of(header);
@@ -1991,13 +2025,13 @@ static uword_t remap_strings(lispobj* where, lispobj* limit, uword_t arg)
                 bitmap = ((struct layout*)native_pointer(layout))->bitmap;
                 for(i=1; i<nwords; ++i)
                     if (layout_bitmap_logbitp(i-1, bitmap))
-                        remap_string(where+i, ht);
+                        coalesce_obj(where+i, ht);
                 continue;
             case CODE_HEADER_WIDETAG:
                 for_each_simple_fun(i, fun, (struct code*)where, 0, {
                     lispobj* fun_slots = SIMPLE_FUN_SCAV_START(fun);
                     for (j=0; j<SIMPLE_FUN_SCAV_NWORDS(fun); ++j)
-                        remap_string(fun_slots+j, ht);
+                        coalesce_obj(fun_slots+j, ht);
                 })
                 nwords = code_header_words(header);
                 break;
@@ -2006,26 +2040,33 @@ static uword_t remap_strings(lispobj* where, lispobj* limit, uword_t arg)
                     continue; // Ignore this object.
             }
             for(i=1; i<nwords; ++i)
-                remap_string(where+i, ht);
+                coalesce_obj(where+i, ht);
         }
     }
     return 0;
 }
 
-void coalesce_strings()
+void coalesce_similar_objects()
 {
     struct hopscotch_table ht;
-    hopscotch_create(&ht, HOPSCOTCH_STRING_HASH, 0, 1<<17, 0);
+    hopscotch_create(&ht, HOPSCOTCH_VECTOR_HASH, 0, 1<<17, 0);
+#ifndef LISP_FEATURE_WIN32
+    // Apparently this triggers the "Unable to recommit" lossage message
+    // in handle_access_violation() in src/runtime/win32-os.c
+    coalesce_range((lispobj*)STATIC_SPACE_START,
+                   (lispobj*)STATIC_SPACE_END,
+                   (uword_t)&ht);
+#endif
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    remap_strings((lispobj*)IMMOBILE_SPACE_START,
-                  (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value,
-                  (uword_t)&ht);
-    remap_strings((lispobj*)IMMOBILE_VARYOBJ_SUBSPACE_START,
-                  (lispobj*)SYMBOL(IMMOBILE_SPACE_FREE_POINTER)->value,
-                  (uword_t)&ht);
+    coalesce_range((lispobj*)IMMOBILE_SPACE_START,
+                   (lispobj*)SYMBOL(IMMOBILE_FIXEDOBJ_FREE_POINTER)->value,
+                   (uword_t)&ht);
+    coalesce_range((lispobj*)IMMOBILE_VARYOBJ_SUBSPACE_START,
+                   (lispobj*)SYMBOL(IMMOBILE_SPACE_FREE_POINTER)->value,
+                   (uword_t)&ht);
 #endif
 #ifdef LISP_FEATURE_GENCGC
-    walk_generation(remap_strings, -1, (uword_t)&ht);
+    walk_generation(coalesce_range, -1, (uword_t)&ht);
 #else
     // FIXME: implement
 #endif
