@@ -20,10 +20,9 @@
 #include "gc-internal.h" // for os_validate()
 #include "hopscotch.h"
 #include <stdint.h>
-#ifdef HOPSCOTCH_INSTRUMENT
 #include <stdio.h>
-#endif
 #include "genesis/vector.h"
+#include "murmur_hash.h"
 
 typedef struct hopscotch_table* tableptr;
 void hopscotch_integrity_check(tableptr,char*,int);
@@ -271,15 +270,50 @@ uword_t sxhash_simple_string(struct vector* string)
     return result;
 }
 
-boolean hopscotch_stringeq(uword_t arg1, uword_t arg2)
+// This works on vector-like objects, which includes most numerics.
+static uword_t vector_sxhash(lispobj* object)
 {
-    lispobj* str1 = (lispobj*)arg1;
-    lispobj* str2 = (lispobj*)arg2;
-    if (((str1[0] ^ str2[0]) & WIDETAG_MASK) == 0 && str1[1] == str2[1]) {
-        sword_t size = sizetab[widetag_of(*str1)](str1);
-        return !memcmp(str1 + 2, str2 + 2, (size-2) << WORD_SHIFT);
-    }
-    return 0;
+    lispobj header = *object;
+    int widetag = widetag_of(header);
+    sword_t nwords = sizetab[widetag](object);
+    // Mix words. At present this works correctly only for specialized vectors,
+    // general vectors with eq-comparable elements,
+    // and numbers that do not contain pointers.
+    return gpr_murmur_hash3(object+1,
+                            (nwords-1) * N_WORD_BYTES,  // exclude header word
+                            // Ignore vector header data bits except for the widetag
+                            widetag >= SIMPLE_ARRAY_WIDETAG
+                              ? hopscotch_hmix(widetag)
+                              : hopscotch_hmix(header & 0xFFFFFFFF));
+}
+
+/* Compare vectors elementwise for EQLness.
+ * This is unlike EQUAL because it works on any specialized vector,
+ * and unlike EQUALP because it compares strings case-sensitively.
+ * Note: this works on most numbers too, not just vectors.
+ */
+static boolean vector_eql(uword_t arg1, uword_t arg2)
+{
+    if (arg1 == arg2) return 1;
+    lispobj* obj1 = (lispobj*)arg1;
+    lispobj* obj2 = (lispobj*)arg2;
+    lispobj header1 = *obj1;
+    if (((header1 ^ *obj2) & WIDETAG_MASK) != 0)
+        return 0; // not eql - different type objects
+
+    int widetag1 = widetag_of(header1);
+    sword_t nwords = sizetab[widetag1](obj1);
+    if (widetag1 < SIMPLE_ARRAY_UNSIGNED_BYTE_2_WIDETAG)
+        // All words must match exactly. Start by comparing the length
+        // (as encoded in the header) since we don't yet know that obj2
+        // occupies the correct number of words.
+        return header1 == *obj2
+            && !memcmp(obj1 + 1, obj2 + 1, (nwords-1) << WORD_SHIFT);
+
+    // Vector elements must have already been coalesced
+    // when comparing simple-vectors for similarity.
+    return (obj1[1] == obj2[1]) // same length vectors
+        && !memcmp(obj1 + 2, obj2 + 2, (nwords-2) << WORD_SHIFT);
 }
 
 /* Initialize 'ht' for first use, which entails zeroing the counters
@@ -296,9 +330,14 @@ void hopscotch_create(tableptr ht, int hashfun,
     case HOPSCOTCH_HASH_FUN_MIX:
       ht->compare = 0; ht->hash = hopscotch_hmix; break;
     case HOPSCOTCH_STRING_HASH:
-      ht->compare = hopscotch_stringeq;
+      ht->compare = vector_eql;
       ht->hash = (uint32_t(*)(uword_t))sxhash_simple_string;
       break;
+    case HOPSCOTCH_VECTOR_HASH:
+      ht->compare = vector_eql;
+      ht->hash = (uint32_t(*)(uword_t))vector_sxhash;
+      break;
+
     default: lose("Bad hash function");
     }
     switch (bytes_per_value) {
@@ -712,16 +751,16 @@ void hopscotch_integrity_check(tableptr ht, char*when, int verbose)
       }
       if (verbose) {
           if (claimed==i || (claimed==-1 && !key))
-            fprintf(s, "      ");
+            fprintf(s, "        ");
           else if (claimed!=-1) {
-            fprintf(s, "[%4d]", claimed);
+            fprintf(s, "[%6d]", claimed);
             if ((int)(ht->mask & hash(ht, key)) != claimed)
               lose("key hashes to wrong logical cell?");
           } else { // should have been claimed
             fprintf(s, " **** ");
             fail = 1;
           }
-          fprintf(s, " %4d: %04x", i, i <= ht->mask ? get_hop_mask(ht,i) : 0);
+          fprintf(s, " %6d: %04x", i, i <= ht->mask ? get_hop_mask(ht,i) : 0);
           if (key) {
             fprintf(s, " %lx -> %d", key, (int)(ht->mask & hash(ht, key)));
             if (ht->values)
